library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;
library grlib, techmap;
use grlib.amba.all;
use grlib.amba.all;
use grlib.stdlib.all;
use techmap.gencomp.all;
use techmap.allclkgen.all;
library gaisler;
use gaisler.memctrl.all;
use gaisler.leon3.all;
use gaisler.uart.all;
use gaisler.misc.all;
library esa;
use esa.memoryctrl.all;
--use gaisler.sim.all;
library lpp;
use lpp.lpp_ad_conv.all;
use lpp.lpp_amba.all;
use lpp.apb_devices_list.all;
use lpp.general_purpose.all;
use lpp.lpp_cna.all;
use lpp.lpp_memory.all;

Library UNISIM;
use UNISIM.vcomponents.all;

use work.config.all;

entity beagleSigGen is
	generic (
			memtech       : integer := CFG_MEMTECH;
			padtech       : integer := CFG_PADTECH;
			clktech       : integer := CFG_CLKTECH
			);
    Port ( 
	        clk          : in  STD_LOGIC;
           rstn         : in  STD_LOGIC;
			  CAL_IN_SCK   : out std_ulogic;
			  DAC_nCS      : out std_ulogic;
           DAC_SDI      : out std_logic_vector(7 downto 0);
			  address      : in  std_logic_vector(18 downto 0);
			  DATA   	   : in  std_logic_vector(15 downto 0);
			  REN_debug  	: out std_logic;
			  WEN				: in  std_logic;
			  FIFO_FULL	   : out std_logic_vector(7 downto 0);
			  FIFO_EMPTY	: out std_logic_vector(7 downto 0)
			  );
end beagleSigGen;

architecture Behavioral of beagleSigGen is

subtype TAB16  is std_logic_vector(15 downto 0);
type FIFOout_t is array(7 downto 0) of TAB16;

signal FIFO_FULL_net	    : std_logic_vector(7 downto 0);
signal FIFO_EMPTY_net 	 : std_logic_vector(7 downto 0);

signal FIFO_WEN 	       : std_logic_vector(7 downto 0);
signal FIFO_REN 	       : std_logic;

signal FIFO_out    : FIFOout_t;

signal DAC_DATA    : CNA_16bit_T(7 downto 0,15 downto 0);
signal smpclk      : std_logic;
signal smpclk_reg  : std_logic;
signal DAC_SDO	    : std_logic;
signal DATA_reg    : std_logic_vector(15 downto 0);

Constant clk_TRIGER_MAX	:	integer	:=	(150000000/(2*4096))+1;
signal   clk_TRIGER	:	integer range 0 to clk_TRIGER_MAX	:=	clk_TRIGER_MAX;
signal	cpt1		   :	integer;
signal   clk_TRIGER_load  : std_logic;

begin



FIFO_FULL	 <= FIFO_FULL_net;
FIFO_EMPTY	 <= FIFO_EMPTY_net;

FIFOlp : FOR I IN 0 to 7 GENERATE
front_fifoN: lpp_fifo
	generic map(
		 tech          =>  memtech,
		 Mem_use       =>  1, --use RAM not CELS
		 DataSz        =>  16,
		 AddrSz        =>  12
		 )
	port map(
		 rstn    => rstn,
		 ReUse   => '0',
		 rclk    => clk,
		 ren     => FIFO_REN,
		 rdata   => FIFO_out(I),
		 empty   => FIFO_EMPTY_net(I),
		 raddr   => open,
		 wclk    => clk,
		 wen     => FIFO_WEN(I),
		 wdata   => DATA_reg,
		 full    => FIFO_FULL_net(I),
		 waddr   => open
	);
END GENERATE;

--FIFOlp : FOR I IN 0 to 7 GENERATE
--front_fifoN: FIFO_pipeline 
--generic map(
--       tech          =>  memtech,
--		 fifoCount     =>  8,
--		 Mem_use       =>  1, --use RAM not CELS
--		 DataSz        =>  16,
--		 abits         =>  10
--    )
--port map(
--       rstn    => rstn,
--		 ReUse   => '0',
--		 rclk    => clk,
--		 ren     => FIFO_REN,
--		 rdata   => FIFO_out(I),
--		 empty   => FIFO_EMPTY_net(I),
--		 raddr   => open,
--		 wclk    => clk,
--		 wen     => FIFO_WEN(I),
--		 wdata   => DATA_reg,
--		 full    => FIFO_FULL_net(I),
--		 waddr   => open
--);
--END GENERATE;


REN_debug	<= FIFO_REN;

process(clk,rstn)
begin
	if rstn = '0' then
		DATA_reg  <= (others => '0');
		FIFO_WEN  <= (others => '1');
		clk_TRIGER <= clk_TRIGER_MAX;
		clk_TRIGER_load <= '0';
	elsif clk'event and clk = '1' then
		if WEN = '0' then
			DATA_reg <= DATA;
			case address(3 downto 0) is 
				when "0000"=>
					FIFO_WEN <= "11111110";
				when "0001"=>
				   FIFO_WEN <= "11111101";
				when "0010"=>
				   FIFO_WEN <= "11111011";
				when "0011"=>
				   FIFO_WEN <= "11110111";
				when "0100"=>
				   FIFO_WEN <= "11101111";
				when "0101"=>
				   FIFO_WEN <= "11011111";
				when "0110"=>
				   FIFO_WEN <= "10111111";
				when "0111"=>
 				   FIFO_WEN <= "01111111";
				when others =>
				   FIFO_WEN <= "11111111";
			end case;
		else
		   FIFO_WEN <= "11111111";
		end if;
		if WEN = '0' then
			if address(3 downto 0) = "1000" then 
				clk_TRIGER <= to_integer(unsigned(DATA));
				clk_TRIGER_load <= '1';
			end if;
		else
			clk_TRIGER_load <= '0';
		end if;
	end if;
end process;

all_bits: FOR I in 15 downto 0 GENERATE
  all_chans: FOR J in 7 downto 0 GENERATE
	  DAC_DATA(J,I) <= FIFO_out(J)(I);
  end GENERATE;
end GENERATE;



process(clk,rstn)
begin
	if rstn = '0' then
		FIFO_REN	<= '1';
		smpclk_reg <= '0';
	elsif clk'event and clk = '1' then
		smpclk_reg <= smpclk; 
		if smpclk = '1' and smpclk_reg = '0' and  FIFO_EMPTY_net = X"00" then
			FIFO_REN <= '0' ;
		else 
			FIFO_REN <= '1';
		end if;
	end if;
end process;


DAC0 :  DAC8581 
    generic map(150,8)
    Port map( 
	        clk =>      clk,
           rstn =>     rstn,
           smpclk =>   smpclk,
           sclk =>     CAL_IN_SCK,
           csn =>		  DAC_nCS,
           sdo =>      DAC_SDI,
           smp_in =>   DAC_DATA
			  );

	 

--smpclk0: Clk_divider 
--    GENERIC map(OSC_freqHz  => 150000000,
--            TargetFreq_Hz   => 256000)
--    PORT map( 
--			  clk          =>  clk,
--           reset        =>  rstn,
--           clk_divided  =>  smpclk 
--			  );

process(rstn,clk)
begin
	if rstn = '0' then
		cpt1			<=	0;
		smpclk		<=	'0';
	elsif clk'event and clk = '1' then
		if cpt1 = clk_TRIGER or clk_TRIGER_load = '1' then
			smpclk	<=	not smpclk;
			cpt1			<=	0;
		else
			cpt1			<=	cpt1 + 1;
		end if;
	end if;
end process;


end Behavioral;

